@@ -46,6 +46,9 @@
 #include "textfile.h"
 #include "oui.h"
 
+/* Unofficial value, might still change */
+#define LE_LINK		0x03
+
 #define for_each_opt(opt, long, short) while ((opt=getopt_long(argc, argv, short ? short:"+", long, NULL)) != -1)
 
 static void usage(void);
@@ -72,6 +75,8 @@ static char *type2str(uint8_t type)
 		return "ACL";
 	case ESCO_LINK:
 		return "eSCO";
+	case LE_LINK:
+		return "LE";
 	default:
 		return "Unknown";
 	}
